# build_result.rb
# Copyright 2013 Doug Ireton

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

module StashNotifier
  class BuildResult

    attr_accessor :key, :name, :url, :description
    attr_reader :state

    VALID_BUILD_STATES = %w{ INPROGRESS SUCCESSFUL FAILED }

    def initialize(args)
      self.state   = args[:state]
      @key         = args[:key]
      @name        = args[:name]
      @url         = args[:url]
      @description = args[:description]
    end

    def state=(new_state)
      new_state = new_state.upcase
      if VALID_BUILD_STATES.include?(new_state)
        @state = new_state
      else
        raise ArgumentError, "'#{new_state}' is not a valid Stash Build state! Valid states are #{VALID_BUILD_STATES}."
      end
    end

    def to_json
      result = {
        state:       state,
        key:         key,
        name:        name,
        url:         url,
        description: description
      }
      result.to_json
    end

  end
end
