# stash_nofier.rb
# Copyright 2013 Doug Ireton

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'faraday'
require 'json'
require 'stash_notifier/exceptions'

module StashNotifier

  class HTTPClient
    STASH_BUILD_STATUS_PATH = 'rest/build-status/1.0/commits'
    DEFAULTS = {
      http_client: Faraday.new
    }.freeze

    attr_accessor :stash_url, :stash_user, :stash_pwd, :git_commit,
                  :build_result, :http_client

    def initialize(args)
      args = DEFAULTS.merge(args)

      @stash_url    = args[:stash_url]
      @stash_user   = args[:stash_user]
      @stash_pwd    = args[:stash_pwd]
      @git_commit   = args[:git_commit]
      @build_result = args[:build_result]
      @http_client  = args[:http_client]
    end

    def to_s
      "StashNotifier connection:
      url: #{stash_url},
      git_commit: #{git_commit},
      user: #{stash_user},
      pwd: ***********,
      build_result: #{build_result}"
    end

    def post_to_stash
      setup_http_client
      response = post_results
      check_errors(response)
      response
    end

    private

    def setup_http_client
      http_client.url_prefix = stash_url
      http_client.path_prefix = STASH_BUILD_STATUS_PATH
      http_client.basic_auth(stash_user, stash_pwd)
      http_client.ssl[:verify] = false
    end

    def post_results
      http_client.post do |req|
        req.url git_commit
        req.headers['Content-Type'] = 'application/json'
        req.body = build_result
      end
    end

    def check_errors(response)
      unless response.success?

        err_msg = get_stash_err(response)

        case response.status
        when 400
          raise StashNotifier::BadRequest, err_msg
        when 401
          raise StashNotifier::Unauthorized, err_msg
        else
          raise StashNotifier::Error, err_msg
        end
      end
    end

    def get_stash_err(response)
      response_body = JSON.parse(response.body)
      response_body['errors'].first['message']
    end

  end
end
